'use strict';

var core = require('@agile-ts/core');
var Vue = require('vue');
var utils = require('@agile-ts/utils');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Vue__default = /*#__PURE__*/_interopDefaultLegacy(Vue);

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function bindAgileInstances(deps, agile, vueComponent, observerType) {
  var depsWithoutIndicator = [];
  var depsWithIndicator;

  if (utils.isValidObject(deps)) {
    depsWithIndicator = formatDepsWithIndicator(deps, observerType);
  } else {
    var response = formatDepsWithNoSafeIndicator(deps, observerType);
    depsWithIndicator = response.depsWithIndicator;
    depsWithoutIndicator = response.depsWithoutIndicator;
  }

  if (depsWithoutIndicator.length > 0) {
    agile.subController.subscribe(vueComponent, depsWithoutIndicator, {
      waitForMount: false
    });
  }

  if (depsWithIndicator) {
    return agile.subController.subscribe(vueComponent, depsWithIndicator, {
      waitForMount: false
    }).props;
  }

  return {};
}

var formatDepsWithNoSafeIndicator = (deps, observerType) => {
  var depsArray = core.extractRelevantObservers(utils.normalizeArray(deps), observerType);
  var depsWithIndicator = {};
  var depsWithoutIndicator = depsArray.filter(dep => dep !== undefined);
  depsWithoutIndicator = depsWithoutIndicator.filter(dep => {
    if (dep && dep['key'] != null) {
      depsWithIndicator[dep['key']] = dep;
      return false;
    }

    return true;
  });
  return {
    depsWithIndicator,
    depsWithoutIndicator
  };
};

var formatDepsWithIndicator = (deps, observerType) => {
  var depsObject = core.extractRelevantObservers(deps, observerType);
  var depsWithIndicator = {};

  for (var key in depsObject) {
    var observer = depsObject[key];
    if (observer != null) depsWithIndicator[key] = observer;
  }

  return depsWithIndicator;
};

var vueIntegration = new core.Integration({
  key: 'vue',
  frameworkInstance: Vue__default["default"],
  updateMethod: (componentInstance, updatedData) => {
    var componentData = componentInstance.$data;

    for (var key of Object.keys(updatedData)) {
      if (Object.prototype.hasOwnProperty.call(componentData, key)) {
        componentData.sharedState[key] = updatedData[key];
      } else {
        Vue__default["default"].set(componentData.sharedState, key, updatedData[key]);
      }
    }
  },
  bind: agile => {
    Vue__default["default"].use({
      install: vue => {
        vue.mixin({
          created: function created() {
            this.$agile = agile;
          },
          methods: {
            bindAgileInstances: function bindAgileInstances$1(deps) {
              var _this$$data;

              return {
                sharedState: _objectSpread2(_objectSpread2({}, (this === null || this === void 0 ? void 0 : (_this$$data = this.$data) === null || _this$$data === void 0 ? void 0 : _this$$data.sharedState) || {}), bindAgileInstances(deps, agile, this))
              };
            },
            bindAgileValues: function bindAgileValues(deps) {
              var _this$$data2;

              return {
                sharedState: _objectSpread2(_objectSpread2({}, (this === null || this === void 0 ? void 0 : (_this$$data2 = this.$data) === null || _this$$data2 === void 0 ? void 0 : _this$$data2.sharedState) || {}), bindAgileInstances(deps, agile, this, 'value'))
              };
            },
            bindAgileOutputs: function bindAgileOutputs(deps) {
              var _this$$data3;

              return {
                sharedState: _objectSpread2(_objectSpread2({}, (this === null || this === void 0 ? void 0 : (_this$$data3 = this.$data) === null || _this$$data3 === void 0 ? void 0 : _this$$data3.sharedState) || {}), bindAgileInstances(deps, agile, this, 'output'))
              };
            }
          }
        });
      }
    });
    return Promise.resolve(true);
  }
});
core.Integrations.addInitialIntegration(vueIntegration);
var vueIntegration$1 = vueIntegration;

module.exports = vueIntegration$1;
//# sourceMappingURL=index.js.map
